package com.bcxin.tenant.open.domains.repositories;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.bcxin.tenant.open.domains.criterias.PoliceIncidentSearchCriteria;
import com.bcxin.tenant.open.domains.criterias.PoliceIncidentsSearchCriteria;
import com.bcxin.tenant.open.domains.criterias.PoliceIncidentRegionReportCriteria;
import com.bcxin.tenant.open.domains.dtos.PoliceIncidentPendingStatisticsDTO;
import com.bcxin.tenant.open.domains.dtos.PoliceIncidentRegionReportDTO;
import com.bcxin.tenant.open.domains.dtos.PoliceIncidentsSosUpdateDTO;
import com.bcxin.tenant.open.domains.entities.PoliceIncidentsEntity;
import com.bcxin.tenant.open.domains.mappers.PoliceIncidentsMapper;
import com.bcxin.tenant.open.infrastructures.EntityCollection;
import com.bcxin.tenant.open.infrastructures.enums.ResourceType;
import com.bcxin.tenant.open.infrastructures.repositories.RepositoryBaseAbstract;
import org.springframework.stereotype.Repository;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.Collection;
import java.util.Collections;
import java.util.stream.Collectors;

@Repository
public class PoliceIncidentsRepositoryImpl extends RepositoryBaseAbstract implements PoliceIncidentsRepository {
    private final PoliceIncidentsMapper policeIncidentsMapper;

    public PoliceIncidentsRepositoryImpl(PoliceIncidentsMapper policeIncidentsMapper) {
        this.policeIncidentsMapper = policeIncidentsMapper;
    }

    @Override
    public PoliceIncidentsEntity getById(Object id) {
        return this.policeIncidentsMapper.selectById((Long) id);
    }

    @Override
    public void insert(PoliceIncidentsEntity data) {
        this.policeIncidentsMapper.insert(data);
    }

    @Override
    public void update(PoliceIncidentsEntity data) {
        this.policeIncidentsMapper.updateById(data);
    }

    @Override
    public EntityCollection<PoliceIncidentsEntity> search(PoliceIncidentsSearchCriteria criteria) {
        LambdaQueryWrapper<PoliceIncidentsEntity> queryWrapper = new LambdaQueryWrapper<>();

        if (!CollectionUtils.isEmpty(criteria.getSuperviseDepartIds())) {
            queryWrapper = queryWrapper.in(PoliceIncidentsEntity::getSuperviseDepartId, criteria.getSuperviseDepartIds());
        }

        if (!CollectionUtils.isEmpty(criteria.getCategories())) {
            queryWrapper = queryWrapper.in(PoliceIncidentsEntity::getCategory, criteria.getCategories()
                    .stream().map(ii -> ii.ordinal()).collect(Collectors.toList()))
            ;
        }

        if (criteria.getLevel() != null) {
            /**
             * 没有使用ordinal(); 查不出来. 这个有点奇怪
             */
            queryWrapper = queryWrapper.eq(PoliceIncidentsEntity::getPoliceEventLevel, criteria.getLevel().ordinal());
        }

        if (criteria.getStatus() != null) {
            queryWrapper = queryWrapper.eq(PoliceIncidentsEntity::getPoliceEventStatus, criteria.getStatus().ordinal());
        }

        IPage<PoliceIncidentsEntity> page = new Page<>(criteria.getPageIndex(), criteria.getPageSize());

        IPage<PoliceIncidentsEntity> policeIncidentsEntityIPage =
                this.policeIncidentsMapper.selectPage(page, queryWrapper);

        return EntityCollection.create(
                policeIncidentsEntityIPage.getRecords(), criteria.getPageSize(),
                policeIncidentsEntityIPage.getTotal()
        );
    }

    @Override
    public void updateSsoStatus(PoliceIncidentsSosUpdateDTO data) {
        this.policeIncidentsMapper.updateSsoStatus(data);
    }

    @Override
    public EntityCollection<PoliceIncidentsEntity> search(PoliceIncidentSearchCriteria criteria) {
        LambdaQueryWrapper<PoliceIncidentsEntity> queryWrapper = new LambdaQueryWrapper<>();

        if (!CollectionUtils.isEmpty(criteria.getSuperDepartIds())) {
            queryWrapper = queryWrapper.in(PoliceIncidentsEntity::getSuperviseDepartId, criteria.getSuperDepartIds());
        }

        if (!CollectionUtils.isEmpty(criteria.getCategories())) {
            Collection<Integer> categories =
                    criteria.getCategories().stream().map(ii -> ii.ordinal()).collect(Collectors.toList());
            queryWrapper = queryWrapper.in(PoliceIncidentsEntity::getCategory, categories);
        }

        if (!criteria.isSupervise()) {
            if (CollectionUtils.isEmpty(criteria.getOtherSpecialPermissionScopeIds())) {
                if (!CollectionUtils.isEmpty(criteria.getManagedProjectIds())) {
                    /**
                     * 针对赛演临保数据, 不同公司需要看到参与此项目的人员的数据
                     */
                    queryWrapper = queryWrapper.and(wrapper ->
                            wrapper.in(PoliceIncidentsEntity::getProjectId, criteria.getManagedProjectIds())
                    );
                }else {
                    queryWrapper = queryWrapper.in(PoliceIncidentsEntity::getOrganizationId, criteria.getOrganizationIds());
                }
            } else {
                /**
                 * 针对内保单位的情况, 它可以查询关联驻勤点和本公司的数据
                 */
                if (!CollectionUtils.isEmpty(criteria.getManagedProjectIds())) {
                    /**
                     * 针对赛演临保数据, 不同公司需要看到参与此项目的人员的数据
                     */
                    queryWrapper = queryWrapper.and(wrapper ->
                            wrapper.in(PoliceIncidentsEntity::getProjectId, criteria.getManagedProjectIds())
                                    .in(PoliceIncidentsEntity::getSecurityStationId, criteria.getOtherSpecialPermissionScopeIds())
                    );
                } else {
                    queryWrapper = queryWrapper.and(wrapper ->
                            wrapper.in(PoliceIncidentsEntity::getOrganizationId, criteria.getOrganizationIds()).or()
                                    .in(PoliceIncidentsEntity::getSecurityStationId, criteria.getOtherSpecialPermissionScopeIds())
                    );
                }
            }
        }

        if (StringUtils.hasLength(criteria.getEmployeeId())) {
            queryWrapper = queryWrapper.eq(PoliceIncidentsEntity::getTenantEmployeeId, criteria.getEmployeeId());
        }

        if (criteria.getBeginTime() != null) {
            queryWrapper = queryWrapper.ge(PoliceIncidentsEntity::getCreatedTime, criteria.getBeginTime());
        }

        if (criteria.getEndTime() != null) {
            queryWrapper = queryWrapper.le(PoliceIncidentsEntity::getCreatedTime, criteria.getEndTime());
        }

        if (StringUtils.hasLength(criteria.getOrganizationName())) {
            queryWrapper = queryWrapper.likeLeft(PoliceIncidentsEntity::getOrganizationName, criteria.getOrganizationName().concat("%"));
        }

        if (StringUtils.hasLength(criteria.getPoliceName())) {
            queryWrapper = queryWrapper.likeLeft(PoliceIncidentsEntity::getName, criteria.getPoliceName().trim().concat("%"));
        }

        if (StringUtils.hasLength(criteria.getReason())) {
            queryWrapper = queryWrapper.likeLeft(PoliceIncidentsEntity::getReason, criteria.getReason().trim().concat("%"));
        }

        if (StringUtils.hasLength(criteria.getEmpIdNumber())) {
            queryWrapper = queryWrapper.likeLeft(PoliceIncidentsEntity::getEmpIdNumber, criteria.getEmpIdNumber().trim().concat("%"));
        }

        if (StringUtils.hasLength(criteria.getContact())) {
            queryWrapper = queryWrapper.likeLeft(PoliceIncidentsEntity::getContact, criteria.getContact().trim().concat("%"));
        }

        if (StringUtils.hasLength(criteria.getStationName())) {
            queryWrapper = queryWrapper.likeLeft(PoliceIncidentsEntity::getSecurityStationName, criteria.getStationName().trim().concat("%"));
        }

        if (StringUtils.hasLength(criteria.getProcessorName())) {
            queryWrapper = queryWrapper.likeLeft(PoliceIncidentsEntity::getProcessedUserName, criteria.getProcessorName().trim().concat("%"));
        }

        if (criteria.getEventType() != null) {
            queryWrapper = queryWrapper.eq(PoliceIncidentsEntity::getPoliceEventType, criteria.getEventType().intValue());
        }

        if (criteria.getLevel() != null) {
            queryWrapper = queryWrapper.eq(PoliceIncidentsEntity::getPoliceEventLevel, criteria.getLevel().ordinal());
        }

        if (criteria.getStatus() != null) {
            queryWrapper = queryWrapper.eq(PoliceIncidentsEntity::getPoliceEventStatus, criteria.getStatus().ordinal());
        }

        if (criteria.getBeginProcessedTime() != null) {
            queryWrapper = queryWrapper.ge(PoliceIncidentsEntity::getProcessedTime, criteria.getBeginProcessedTime());
        }

        if (criteria.getEndProcessedTime() != null) {
            queryWrapper = queryWrapper.le(PoliceIncidentsEntity::getProcessedTime, criteria.getEndProcessedTime());
        }

        if (StringUtils.hasLength(criteria.getEmpName())) {
            queryWrapper = queryWrapper.likeLeft(PoliceIncidentsEntity::getEmpName, criteria.getEmpName().trim().concat("%"));
        }
        if (StringUtils.hasLength(criteria.getProjectName())) {
            queryWrapper = queryWrapper.likeLeft(PoliceIncidentsEntity::getProjectName, criteria.getProjectName().trim().concat("%"));
        }

        if(StringUtils.hasLength(criteria.getRegionName())) {
            queryWrapper = queryWrapper.apply(String.format(" extend_json_info->'$.regionName'='%s'", criteria.getRegionName().replace("'", "''")));
        }

        /**
         * 针对临保项目的查询, 直接仅根据项目进行过滤
         */
        if (!CollectionUtils.isEmpty(criteria.getManagedProjectIds())) {
            if (criteria.getDeskTypes() != null) {
                int calculatedResourceTypeValue = ResourceType.getCalculatedResourceValueByDeskTypes(criteria.getDeskTypes());
                queryWrapper = queryWrapper.apply("calculated_resource_type&" + calculatedResourceTypeValue + ">0");
            }
        }

        queryWrapper = queryWrapper.orderByDesc(PoliceIncidentsEntity::getCreatedTime);

        if (!criteria.ignorePage()) {
            IPage<PoliceIncidentsEntity> page = new Page<>(criteria.getPageIndex(), criteria.getPageSize());

            IPage<PoliceIncidentsEntity> policeIncidentsEntityIPage =
                    this.policeIncidentsMapper.selectPage(page, queryWrapper);

            return EntityCollection.create(
                    policeIncidentsEntityIPage.getRecords(),
                    (int) page.getSize(),
                    policeIncidentsEntityIPage.getTotal()
            );
        } else {
            /**
             * 按照分页取出数据
             */
            queryWrapper = queryWrapper.last(String.format("limit %s, %s ", criteria.getSkip(), criteria.getPageSize()));
            Collection<PoliceIncidentsEntity> policeIncidentsEntityIPage =
                    this.policeIncidentsMapper.selectList(queryWrapper);

            return EntityCollection.createForNoTotalCount(
                    policeIncidentsEntityIPage,criteria.getPageSize()
            );
        }
    }

    @Override
    public Collection<PoliceIncidentPendingStatisticsDTO> getPendingTotal(Collection<String> superviseDepartIds) {
        if (CollectionUtils.isEmpty(superviseDepartIds)) {
            return Collections.EMPTY_LIST;
        }

        return this.policeIncidentsMapper.getPendingTotal(superviseDepartIds);
    }

    @Override
    public void delete(PoliceIncidentsEntity data) {
        this.policeIncidentsMapper.deleteById(data);
    }

    @Override
    public Collection<PoliceIncidentRegionReportDTO> getReport(PoliceIncidentRegionReportCriteria criteria) {
        if(!StringUtils.hasLength(criteria.getRegionName())) {
            return Collections.EMPTY_LIST;
        }

        return this.policeIncidentsMapper.getReport(criteria);
    }
}
