package com.bcxin.tenant.open.domains.repositories;

import com.bcxin.tenant.open.domains.dtos.SyncDataWrapperDTO;
import com.bcxin.tenant.open.domains.dtos.SyncDbCheckQueryDTO;
import com.bcxin.tenant.open.domains.mappers.RdSyncDataMapper;
import com.bcxin.tenant.open.domains.mappers.RdSyncEventProjectDataMapper;
import com.bcxin.tenant.open.domains.mappers.dtos.SyncDataRequestWrapperDTO;
import com.bcxin.tenant.open.infrastructures.constants.BusinessConstants;
import com.bcxin.tenant.open.infrastructures.exceptions.BadTenantException;
import com.bcxin.tenant.open.infrastructures.exceptions.NotSupportTenantException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.Collection;
import java.util.stream.Collectors;

@Repository
public class RdSyncDataRepositoryImpl implements RdSyncDataRepository{
    private static final Logger logger = LoggerFactory.getLogger(RdSyncDataRepositoryImpl.class);

    private final RdSyncDataMapper syncDataMapper;
    private final RdSyncEventProjectDataMapper syncEventProjectDataMapper;

    public RdSyncDataRepositoryImpl(RdSyncDataMapper syncDataMapper,
                                    RdSyncEventProjectDataMapper syncEventProjectDataMapper) {
        this.syncDataMapper = syncDataMapper;
        this.syncEventProjectDataMapper = syncEventProjectDataMapper;
    }

    /**
     * 针对此同步功能，采用读提交的隔离级别来实现; 如下锁的问题
     * INSERT ... ON DUPLICATE KEY UPDATE differs from a simple INSERT in that an exclusive lock rather than a shared lock is placed on the row to be updated when a duplicate-key error occurs. An exclusive index-record lock is taken for a duplicate primary key value. An exclusive next-key lock is taken for a duplicate unique key value.
     *
     * REPLACE is done like an INSERT if there is no collision on a unique key. Otherwise, an exclusive next-key lock is placed on the row to be replaced.
     *
     * INSERT INTO T SELECT ... FROM S WHERE ... sets an exclusive index record lock (without a gap lock) on each row inserted into T. If the transaction isolation level is READ COMMITTED, or innodb_locks_unsafe_for_binlog is enabled and the transaction isolation level is not SERIALIZABLE, InnoDB does the search on S as a consistent read (no locks). Otherwise, InnoDB sets shared next-key locks on rows from S. InnoDB has to set locks in the latter case: During roll-forward recovery using a statement-based binary log, every SQL statement must be executed in exactly the same way it was done originally.
     *
     * CREATE TABLE ... SELECT ... performs the SELECT with shared next-key locks or as a consistent read, as for INSERT ... SELECT.
     *
     * When a SELECT is used in the constructs REPLACE INTO t SELECT ... FROM s WHERE ... or UPDATE t ... WHERE col IN (SELECT ... FROM s ...), InnoDB sets shared next-key locks on rows from table s.
     * @param dataWrapper
     */
    @Transactional(isolation = Isolation.READ_UNCOMMITTED)
    @Override
    public void reloadSyncData(SyncDataWrapperDTO dataWrapper) {
        SyncDataRequestWrapperDTO data =
                SyncDataRequestWrapperDTO.create(dataWrapper.getIds(),dataWrapper.getAdditionalParameter());

        switch (dataWrapper.getDataType()) {
            case User -> {
                if (!CollectionUtils.isEmpty(dataWrapper.getIds())) {
                    this.syncDataMapper.reloadSyncUsers(data);
                }
            }
            case Employee -> {
                this.syncDataMapper.reloadSyncEmployees(data);
            }
            case Station -> {
                this.syncDataMapper.reloadSyncStations(data);
                this.syncEventProjectDataMapper.reloadSyncPostManagersByResourceId(data);

                logger.error("刷新驻勤点信息:ids={}, additionalParameter={}",
                        data.getIds().stream().collect(Collectors.joining()), data.getAdditionalParameter()
                );
                this.syncDataMapper.reloadWorkstationsBySecurityStation(data);
            }
            case Company -> {
                this.syncDataMapper.reloadSyncCompanies(data);
            }
            case ExamRoom -> {
                this.syncDataMapper.reloadSyncExamSiteRooms(data);
            }
            case ExamSite -> {
                this.syncDataMapper.reloadSyncExamSites(data);
            }
            case SystemExamInfo -> {
                this.syncDataMapper.reloadSyncExamInfo(data);
            }
            case RefreshCompanyDesks -> {
                this.syncDataMapper.reloadSyncCompanyDesks(data);
            }
            case RefreshCompanyPoints -> {
                this.syncDataMapper.reloadSyncCompanyPoints(data);
            }
            case CommunityUser -> {
                this.syncDataMapper.reloadCommunityUsers(data);
            }
            case SuperviseDeparts -> {
                this.syncDataMapper.reloadSuperviseDeparts(data);
            }
            case ProprietorStations -> {
                this.syncDataMapper.reloadProprietorStations(data);
            }
            case CompanyPermissions -> {
                this.syncDataMapper.reloadCompanyPermissions(data);
            }
            case TemporaryProtectionProject -> {
                Collection<String> ids = dataWrapper.getIds();
                if (CollectionUtils.isEmpty(ids)) {
                    return;
                }
                Collection<String> tempProjIds = ids.stream()
                        .filter(ii -> !BusinessConstants.isRelativeProjectIdStuffId(ii)).collect(Collectors.toSet());
                if (!CollectionUtils.isEmpty(tempProjIds)) {
                    this.syncDataMapper.reloadTemporaryProtectionProject(SyncDataRequestWrapperDTO.create(
                            tempProjIds,
                            tempProjIds
                    ));
                }
                Collection<String> eventProjectIds = new ArrayList<>();
                if (!CollectionUtils.isEmpty(ids)) {
                    eventProjectIds = ids.stream().filter(ii -> BusinessConstants.isRelativeProjectIdStuffId(ii)).collect(Collectors.toSet());
                }
                if (!CollectionUtils.isEmpty(eventProjectIds)) {
                    SyncDataRequestWrapperDTO eventProjectData =
                            SyncDataRequestWrapperDTO.create(
                                    eventProjectIds,
                                    eventProjectIds
                            );

                    this.syncEventProjectDataMapper.reloadSyncProjects(eventProjectData);
                    this.syncEventProjectDataMapper.reloadSyncPostManagersByResourceId(data);
                    this.syncEventProjectDataMapper.reloadSyncTeamsByProjectId(data);
                    this.syncEventProjectDataMapper.reloadSyncPostManagersByProjectId(data);
                }
            }
            case TemporaryProtectionProjectGroup -> {
                this.syncDataMapper.reloadTemporaryProjectGroup(data);
            }
            case TemporaryProtectionProjectMember -> {
                this.syncDataMapper.reloadTemporaryProjectMember(data);
            }
            case HareWareDevice, Device -> {
                this.syncDataMapper.reloadHardwareDevices(data);
            }
            case TemporaryProtectionProjectPersonType -> {
                this.syncDataMapper.reloadTemporaryProjectPersonTypes(data);
            }
            case EventOrganizerJoinProject -> {
                this.syncDataMapper.reloadEventOrgPermissions(data);
                this.syncEventProjectDataMapper.reloadSyncTeams(data);
            }
            case EventOrganizerStationManager -> {
                this.syncDataMapper.reloadEventOrgEmpPermissions(data);
                this.syncEventProjectDataMapper.reloadSyncPostManagers(data);
            }
            case EventOrganizerLimitedResource -> {
                this.syncEventProjectDataMapper.reloadSyncTeamsLimitedResources(data);
            }
            case DispatchDataSource -> {
                this.syncDataMapper.reloadDispatchDataSources(data);
            }
            case Member -> {
                this.syncDataMapper.reloadMembers(data);
            }
            case Workstation -> {
                this.syncDataMapper.reloadWorkstations(data);
            }
            case WorkStationUsers -> {
                this.syncDataMapper.reloadWorkstationUsers(data);
            }
            case WorkStationUsersBySecurityStation -> {
                this.syncDataMapper.reloadWorkstationUsersBySecurityStation(data);
            }
            case VideoDevice -> {
                this.syncDataMapper.reloadStationVideos(data);
            }
            default -> {
                throw new NotSupportTenantException("不支持该数据类型");
            }
        }
    }

    @Override
    public void clearExpiredProprietorStations(Collection<String> ids) {
        if (CollectionUtils.isEmpty(ids)) {
            throw new BadTenantException("参数不能为空");
        }

        this.syncDataMapper.clearExpiredProprietorStations(ids);
    }

    @Override
    public Collection<String> getRelativeIds(SyncDbCheckQueryDTO criteria) {
        return this.syncDataMapper.getRelativeIds(criteria);
    }
}
